# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypfop', 'pypfop.templates']

package_data = \
{'': ['*'], 'pypfop': ['skeletons/mako/*']}

install_requires = \
['Mako>=1.1.4,<2.0.0',
 'cssselect>=1.1.0,<2.0.0',
 'cssutils>=1.0.2,<2.0.0',
 'lxml>=4.6.2,<5.0.0']

setup_kwargs = {
    'name': 'pypfop',
    'version': '1.0a1',
    'description': 'Document preprocessor for Apache FOP',
    'long_description': '.. -*- rst -*-\n\nPython Preprocessor of the Formatting Objects Processor [pypfop]\n================================================================\n\nGenerate PDF files [1]_ based on `Apache FOP`_.\n\nInstallation\n------------\n\n1. Install pypfop::\n\n    pip install pypfop\n\n2. Install `Apache FOP`_:\n\n   #. Download the binary package of fop (preferably 2.6) either the  zip_ or tar_ package.\n   #. Decompress anywhere you like and set the environment variable ``FOP_CMD``\n      to the executable file ``fop`` on the decompressed folder or have the ``fop`` command\n      in your ``PATH``. [2]_\n\nQuick start\n-----------\n\nCreate a file named ``helloworld.fo.mako``:\n\n.. code-block:: mako\n\n  <%inherit file="letter-portrait.fo.mako" />\n  <block>Hello ${name}!</block>\n\n\nAnd then in you python code:\n\n.. code-block:: python\n\n  import pypfop\n\n  pdf_path = pypfop.generate_document("helloworld.fo.mako", {"name": "Foo"})\n  print("The document has been generated at {}".format(pdf_path))\n\nAlternatively there is decorator based syntax:\n\n.. code-block:: python\n\n   import pypfop\n\n   document = pypfop.make_document_decorator()\n\n   @document(template="helloworld.fo.mako")\n   def hello_world():\n       return {"name": "Foo"}\n\n   pdf_path = hello_world()\n\n   print("The document has been generated at {}".format(pdf_path))\n\n\nHow does it work?\n-----------------\n\nIt does what the huge title is implying, preprocess a *higher* level template\nto generate *dynamically* an specific `XSL-FO`_ document, which then gets\nfed to `Apache FOP`_ and generate the expected output. So that means that\nthis packages *requires Java*  ``>_<\'``, but fear not! It is almost transparent\nto the python application.\n\nIn general the internal workflow is::\n\n    template ->  mako -> apply css ->  xsl-fo ->  fop -> *Document*\n\n\n\nUsage\n-----\n\nThe Markup\n^^^^^^^^^^\n\nThe markup used to generate the documents is almost the same as the xsl-fo,\nthe only difference is that is not necessary to set the xml namespace to all\nthe elements, for example:\n\n.. code-block:: xml\n\n  <fo:table>\n   <fo:table-header>\n      <fo:table-row>\n         <fo:table-cell>\n           <fo:block>Project</fo:block>\n         </fo:table-cell>\n      </fo:table-row>\n   </fo:table-header>\n   <fo:table-body>\n      <fo:table-row>\n        <fo:table-cell>\n           <fo:block>pypfop</fo:block>\n       </fo:table-cell>\n      </fo:table-row>\n   </fo:table-body>\n  </fo:table>\n\ncan be written like this:\n\n.. code-block:: xml\n\n  <table>\n   <table-header>\n      <table-row>\n         <table-cell>\n           <block>Project</block>\n         </table-cell>\n      </table-row>\n   </table-header>\n   <table-body>\n      <table-row>\n        <table-cell>\n           <block>pypfopp</block>\n       </table-cell>\n      </table-row>\n   </table-body>\n  </table>\n\n\nThe higher level template language\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n\nCurrently the only supported template language is mako_. If for\nsome reason you don\'t like that templating language, it shouldn\'t be\nhard to extend to your favorite template language based in the implementation\nof mako (which is pretty straight forward) and hopefully contribute back\nto the project :).\n\nFor example, the previous table can be generated with this mako template\nassuming the `header` and `rows` variables are passed as parameters:\n\n\n.. code-block:: mako\n\n    <table>\n      <table-header>\n        <table-row>\n          % for name in header:\n          <table-cell>\n            <block>${name}</block>\n          </table-cell>\n          % endfor\n        </table-row>\n      </table-header>\n      <table-body>\n        % for row in rows:\n           <table-row>\n         % for cell in row:\n            <table-cell>\n              <block>${cell}</block>\n            </table-cell>\n         % endfor\n           </table-row>\n        % endfor\n      </table-body>\n    </table>\n\n\nSkeletons\n^^^^^^^^^\n\nThe previous examples are just fragments of a document. To be able to fully\ngenerate a document with metadata, paper size, margins, etc and to avoid the\nrepetitive work to write this kind of *base document* pypfop have the notion of\n*skeleton documents*, the purpose if this documents are to be inherited at each\ntemplate, at the time the implemented skeleton are:\n\n - ``pypfop/skeletons/mako/A4-landscape.fo.mako``\n - ``pypfop/skeletons/mako/A4-portrait.fo.mako``\n - ``pypfop/skeletons/mako/letter-landscape.fo.mako``\n - ``pypfop/skeletons/mako/letter-portrait.fo.mako``\n\n\nThose include place-holders for:\n\nMetadata:\n\n - title\n - author\n - subject\n - GENERATOR (by default "PyPFOP")\n\nThere is also a  mako block called ``rfooter`` and the body of your template will\nbe the body of the document.\n\nYou don\'t have to define anything else than the body of your own document but you\nstill have the option to override any of the metadata and your own footer region.\n\nTo be a fully functional template for pypfop the previous table need to be like this.\n\n\n``simple-table.fo.mako``:\n\n.. code-block:: mako\n\n    <%inherit file="simple-letter-base.fo.mako" />\n    <table id="main-table">\n      <table-header>\n        <table-row>\n          % for name in header:\n          <table-cell>\n            <block>${name}</block>\n          </table-cell>\n          % endfor\n        </table-row>\n      </table-header>\n      <table-body>\n        % for row in rows:\n           <table-row>\n         % for cell in row:\n            <table-cell>\n              <block>${cell}</block>\n            </table-cell>\n         % endfor\n           </table-row>\n        % endfor\n      </table-body>\n    </table>\n\n\n*The skeletons directory is set in the template directory path by default.*\n\n\nFormat and style with CSS\n^^^^^^^^^^^^^^^^^^^^^^^^^\n\nBeside the *higher level language* that define the content and layout of\nthe document, the style and formatting uses *CSS*, to be more specific it\ncan parse the rules that cssutils_ support, which are a very good subset\nof CSS2 and CSS3, for example it support things like ``:nth-child(X)``\nand ``@import url(XX)``.\n\nThe properties that can be set are the same as in the specification of xsl-fo,\ncheck out the section of `About XSL-FO syntax`_, with the only exception\nthat you can use classes as selectors, xsl-fo does not support the\n``class`` attribute, the pypfop parser is going to look for the\n``class`` attribute then substitute with the specific style and then remove\nthe ``class`` attribute.\n\nFor example I could define the style for the previous table in three files.\n\n*simple_table.css*:\n\n.. code-block:: css\n\n    @import url("general.css");\n    @import url("colors.css");\n\n    #main-table > table-header > table-row{\n        text-align: center;\n        font-weight: bold;\n    }\n\n    #main-table > table-header table-cell{\n        padding: 2mm 0 0mm;\n    }\n\n\n*general.css*:\n\n.. code-block:: css\n\n    flow[flow-name="xsl-region-body"] {\n        font-size: 10pt;\n        font-family: Helvetica;\n    }\n\n*colors.css*:\n\n.. code-block:: css\n\n    #main-table> table-body > table-row > table-cell:first-child{\n        color: red;\n    }\n    #main-table> table-body > table-row > table-cell:nth-child(2){\n        color: blue;\n    }\n    #main-table> table-body > table-row > table-cell:nth-child(3){\n        color: cyan;\n    }\n    #main-table> table-body > table-row > table-cell:last-child{\n        color: green;\n    }\n\n\nGenerate the document\n^^^^^^^^^^^^^^^^^^^^^\n\nThere are a few different ways to generate a document.\n\n\nSingle function call\n%%%%%%%%%%%%%%%%%%%%\n\n.. code-block:: python\n\n  import pypfop\n\n  params = {\n     \'header\': [\'Project\', \'Website\', \'Language\', \'Notes\'],\n     \'rows\': [\n       (\'pypfop\', \'https://github.com/cyraxjoe/pypfop\',\n        \'Python\', \'Abstraction on top of Apache FOP\'),\n       (\'Apache FOP\', \'https://xmlgraphics.apache.org/fop/\',\n       \'Java\', \'\')\n     ]\n  }\n\n  doc_path = pypfop.generate_document(\n      "sample-table.fo.mako",\n      params,\n      "simple_table.css"\n  ) # returns the path of the generated file.\n\n  print(doc_path)\n\n\n\nDecorator based\n%%%%%%%%%%%%%%%\n\n.. code-block:: python\n\n  import pypfop\n\n  document = pypfop.make_document_decorator()\n\n  @document("simple-table.fo.mako", "simple_table.css")\n  def simple_table():\n      return {\n         \'header\': [\'Project\', \'Website\', \'Language\', \'Notes\'],\n         \'rows\': [\n           (\'pypfop\', \'https://github.com/cyraxjoe/pypfop\',\n            \'Python\', \'Abstraction on top of Apache FOP\'),\n           (\'Apache FOP\', \'https://xmlgraphics.apache.org/fop/\',\n           \'Java\', \'\')\n         ]\n       }\n\n  doc_path = simple_table() # returns the path of the generated file.\n\n  print(doc_path)\n\n\n\nExplicit construction\n%%%%%%%%%%%%%%%%%%%%%%\n\n.. code-block:: python\n\n  import pypfop\n  import pypfop.templates.mako\n\n  tfactory = pypfop.templates.mako.Factory()\n  params = {\n    \'header\': [\'Project\', \'Website\', \'Language\', \'Notes\'],\n    \'rows\': [\n      (\'pypfop\', \'https://github.com/cyraxjoe/pypfop\',\n       \'Python\', \'Abstraction on top of Apache FOP\'),\n      (\'Apache FOP\', \'https://xmlgraphics.apache.org/fop/\', \'Java\', \'\')\n    ]\n  }\n  doc_gen = pypfop.DocumentGenerator(tfactory(\'simple-table.fo.mako\'), \'simple_table.css\')\n  doc_path = doc_gen.generate(params) # returns the path of the generated file.\n  print(doc_path)\n\n\nSupported document formats\n^^^^^^^^^^^^^^^^^^^^^^^^^^\n\nIn the previous example we didn\'t define the output of the ``Document`` in\nthat case the default output of ``pdf`` is used, but the supported outputs\nare the almost the same as in `Apache FOP output formats`_.\n\n * pdf\n * rtf\n * tiff\n * png\n * pcl\n * ps\n * txt\n\n\nThe output format can be set in any of the supported methods:\n\n.. code-block:: python\n\n  # simple function call\n  pypfop.generate_document(\n      "sample-table.fo.mako", "simple_table.css", out_format=\'rtf\'\n  )\n\n  # decorator based\n  @document("simple-table.fo.mako", "simple_table.css", out_format=\'rtf\')\n  def simple_table(): ...\n\n\n  # explicit method\n  doc_gen.generate(params, out_format=\'rtf\')\n\n\nAbout XSL-FO syntax\n^^^^^^^^^^^^^^^^^^^\n\nAs you may have already noticed, it is required to know how to format xsl-fo\ndocuments which in most part are very similar to the HTML counterparts\n(except that anything needs to be in ``block`` tags), two of the best reference\nthat I could find online is in the `XML Bible`_ and the `Data 2 Type tutorial`_.\n\nHow about a CSS pre-processor and base generic styles?\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n\nI\'m looking to add support for less_ or something similar and try to generalize\nthe styling of fonts, colors, tables, etc. Very much in the *bootstrap* sense\nof the things. If you are interested in something similar we can join forces\nand build something nice.\n\nWhy!\n----\n\nThe project used to be part of a larger project of one of my clients,\non which I decide early on that I will *only use python 3*, terrible decision\nif you want to generate pdf files easily or at least at the time (2012 I believe?)\nwhen the `Report Lab PDF Toolkit`_ was not yet available for Python 3 and I was looking\nto have some kind of *template* to the very rigid format of the average invoice\nand billing order, so pypfop came to relieve that pain.\n\n.. [1] Actually... you can generate more than PDFs as you will discover if you continue reading.\n.. [2] Actually... you can set the command at another level, check the ``pypfop.document_generator.DocumentGenerator`` class.\n\n.. _`Apache FOP`: https://xmlgraphics.apache.org/fop/\n.. _XSL-FO: https://en.wikipedia.org/wiki/XSL_Formatting_Objects\n.. _zip: https://www.apache.org/dyn/closer.cgi?filename=/xmlgraphics/fop/binaries/fop-2.6-bin.zip&action=download\n.. _tar: https://www.apache.org/dyn/closer.cgi?filename=/xmlgraphics/fop/binaries/fop-2.6-bin.tar.gz&action=download\n.. _`XML Bible`: http://www.ibiblio.org/xml/books/bible3/chapters/ch16.html\n.. _mako: http://www.makotemplates.org/\n.. _cssutils: http://pypi.python.org/pypi/cssutils\n.. _`Apache FOP output formats`: https://xmlgraphics.apache.org/fop/2.6/output.html\n.. _`Data 2 Type tutorial`: http://www.data2type.de/en/xml-xslt-xslfo/xsl-fo/\n.. _`Report Lab PDF Toolkit`: https://pypi.org/project/reportlab/\n.. _less: http://lesscss.org/1\n',
    'author': 'Joel Rivera',
    'author_email': 'rivera@joel.mx',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/cyraxjoe/pypfop',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
